<?php
/**
 * Gradually pushes all imagingsession images to a remote server.
 * Will look for newer inspections than those already done, if not found will look for older. Intent is to
 * prioritise pushing new inspections over backfill of historical images.
 */

$safety=6; //Don't import the last X inspections - might not be complete.
$perRun=10; //How many sets of images to copy at once.
$storePath='/icebearstore/imagestore/'; //local
$domain='http://195.148.31.76'; //destination
$maxTarMegabytes=6;
chdir(__DIR__);

if(file_exists('lockfile')){ die('Lock file exists, aborting'); }
touch('lockfile');

set_time_limit(0);
ini_set('memory_limit','1024M');

$isFirstRun=false;
$maxImagingSessionId=@file_get_contents('maxImagingSessionId');
$minImagingSessionId=@file_get_contents('minImagingSessionId');
if(false===$maxImagingSessionId && false===$minImagingSessionId){
    $isFirstRun=true;
}
if(false===$maxImagingSessionId){ $maxImagingSessionId=999999999999; }
if(false===$minImagingSessionId){ $minImagingSessionId=0; }
$maxImagingSessionId=(int)$maxImagingSessionId;
$minImagingSessionId=(int)$minImagingSessionId;

//define class autoloader
spl_autoload_register(function($className){
    global $projectWebsiteEnabled;
    $paths=array(
        '../../classes/',
        '../../classes/core/',
        '../../classes/core/exception/',
        '../../classes/core/authentication/',
        '../../classes/core/interface/',
        '../../classes/model/',
    );
    foreach($paths as $path){
        if(file_exists($path.$className.'.class.php')){
            /** @noinspection PhpIncludeInspection */
            include_once($path.$className.'.class.php');
        }
    }
});


//connect to the database

try {
    database::connect();
    database::begin();
    session::init(new DummySession());
    session::becomeAdmin();
    Log::init(Log::LOGLEVEL_DEBUG);
    if($isFirstRun) {
        Log::write(Log::LOGLEVEL_INFO, 'First run');
        $query='SELECT id FROM imagingsession ORDER BY id DESC LIMIT '.$safety.','.$perRun.'';
        $result=database::queryGetAll($query);
        if(null===$result) {
            $result=array();
            $count=0;
        } else {
            $result=$result['rows'];
            $result=array_column($result, 'id');
            $count=count($result);
            $maxImagingSessionId=(int)($result[0]);
            $minImagingSessionId=(int)($result[$count-1]);
        }
    } else {
        $query='SELECT id FROM imagingsession WHERE id>'.$maxImagingSessionId.' ORDER BY id DESC LIMIT '.$safety.','.$perRun.'';
        $result=database::queryGetAll($query);
        if(null===$result) {
            $result=array();
            $count=0;
        } else {
            $result=$result['rows'];
            $result=array_column($result, 'id');
            $count=count($result);
        }
        Log::write(Log::LOGLEVEL_INFO, 'Found '.$count.' newer');
        if($count<$perRun){
            $query='SELECT id FROM imagingsession WHERE id<'.$minImagingSessionId.' ORDER BY id DESC LIMIT '.$perRun.'';
            $result2=database::queryGetAll($query);
            if(null===$result2){
                Log::write(Log::LOGLEVEL_INFO, 'Found no older');
            } else {
                $result2=array_column($result2, 'id');
                Log::write(Log::LOGLEVEL_INFO, 'Found '.count($result2).' older');
                if(!empty($result2)){
                    $result=array_merge($result, $result2);
                }
            }
        }
    }


    if(empty($result)){
        Log::write(Log::LOGLEVEL_WARN, 'Nothing to do');
    } else {

        $sql='SELECT d1.imagingsessionid AS imagingsessionid, d1.imagepath, d1.thumbnailpath, d1.imagestorepath
                    FROM dropimage AS d1 
                    WHERE d1.imagestorepath="'.$storePath.'"
                    AND d1.imagingsessionid IN('.implode(',',$result).')
                    AND d1.imagepath<>"dummyimage.jpg"
                    AND d1.imagepath<>"dummythumb.jpg"
                    ORDER BY d1.imagepath';

        $images=database::queryGetAll($sql);

        if(null==$images){
            Log::write(Log::LOGLEVEL_WARN,'No images for these imaging sessions');
        } else {
            $images = $images['rows'];
            Log::write(Log::LOGLEVEL_INFO, count($images) . ' images found');

            //Package up the images into tar files, subject to filesize limit
            $count = 0;
            $maxTarBytes = $maxTarMegabytes * 1024 * 1024;
            foreach ($images as $image) {
                if (filesize($image['imagestorepath'] . ' ' . $image['imagepath']) + filesize($image['imagestorepath'] . ' ' . $image['thumbnailpath']) + filesize('images_' . $count . 'tar') > $maxTarBytes) {
                    Log::write(Log::LOGLEVEL_INFO, 'images_'.$count.'.tar is '.filesize('images_'.$count.'.tar').' bytes');
                    $count++;
                }
                system('tar -rf images_' . $count . '.tar -C ' . $image['imagestorepath'] . ' ' . $image['imagepath']);
                system('tar -rf images_' . $count . '.tar -C ' . $image['imagestorepath'] . ' ' . $image['thumbnailpath']);
            }

            //Send the TARs file to the other server
            $count = 0;
            while(file_exists('images_'.$count.'.tar')){

                $delimiter = '-------------' . uniqid();
                $fileFields = array(
                    'images' => array(
                        'type' => 'application/x-tar',
                        'content' => file_get_contents('images_' . $count . '.tar')
                    )
                );
                $data = '';
                foreach ($fileFields as $name => $file) {
                    $data .= $file['content'] . "\r\n";
                }
                $handle = curl_init($domain . "/api/imagepush/receive.php");
                curl_setopt($handle, CURLOPT_POST, true);
                curl_setopt($handle, CURLOPT_HTTPHEADER, array(
                    'Content-Type: multipart/form-data; boundary=' . $delimiter,
                    'Content-Length: ' . strlen($data)));
                curl_setopt($handle, CURLOPT_POSTFIELDS, $data);
                curl_exec($handle);
                @unlink('images_' . $count . '.tar');
            }
        }

        $maxImagingSessionId=max((int)$result[0], $maxImagingSessionId);
        $minImagingSessionId=min((int)($result[count($result)-1]), $minImagingSessionId);
        echo $maxImagingSessionId.' '.$minImagingSessionId;
        $fh1=@fopen('maxImagingSessionId','wb');
        if(!$fh1 || false===@fwrite($fh1, "".$maxImagingSessionId) || !@fclose($fh1)){
            throw new Exception('Could not write max file');
        }
        $fh2=@fopen('minImagingSessionId','wb');
        if(!$fh2 || false===@fwrite($fh2, "".$minImagingSessionId) || !@fclose($fh2)){
            throw new Exception('Could not write min file');
        }
    }


    @unlink('lockfile');

} catch (Exception $e){
    var_dump($e);
}

database::abort();
