<?php class screenfromrockmaker extends screen {

    protected static $fields=array(
        'name'=>validator::REQUIRED,
        'manufacturer'=>validator::REQUIRED,
        'catalognumber'=>validator::ANY,
        'path'=>validator::REQUIRED
    );

    private static $baseUri='https://formulatrix.com/screens/';

    public static function getAll($request=array()){
        $screens=session::get('RockMakerScreens');
        if(!$screens){
            $screens=static::getRockMakerScreens();
            session::set('RockMakerScreens', $screens);
        }

        foreach ($screens as &$manufacturer){
            if(!isset($manufacturer['screens'])){
                continue;
            }
            foreach ($manufacturer['screens'] as &$screen){
                $existing=screen::getByProperties(array(
                    'manufacturer'=>$manufacturer['manufacturer'],
                    'name'=>$screen['name']
                ), $request);
                if(isset($existing['rows'])){
                    $screen['installed']=1;
                    $screen['id']=$existing['rows'][0]['id'];
                } else {
                    $screen['installed']=0;
                }
            }
        }

        return array(
            'total'=>count($screens),
            'rows'=>$screens
        );
    }

    public static function create($request=array()){

        //try to create, will fail if already exists

        $url=static::$baseUri.str_replace(' ','%20',$request['path']);
        $parts=explode('/', $request['path']);
        $tempDir=rtrim(config::get('core_imagestore'),'/').'/screens_temp/'.$parts[0].'/'.$parts[1];
        $fileName=end($parts);
        $zipFile=$tempDir.'/'.$fileName;
        $cleanUp=[];

        try {
            if(!file_exists($tempDir) && !@mkdir($tempDir, 0777, true)){
                throw new ServerException('Could not create temporary directory on server');
            }
            $cleanUp[]=$zipFile;
            $out=@fopen($zipFile,"wb");
            if(false===$out){
                throw new ServerException('Could not download update because destination path '.$zipFile.' is not writable.');
            }
            $ch=curl_init();
            curl_setopt($ch, CURLOPT_FILE, $out);
            curl_setopt($ch, CURLOPT_URL, $url);
            curl_setopt($ch, CURLOPT_FOLLOWLOCATION, true);
            curl_setopt($ch,CURLOPT_SSL_VERIFYPEER, false);
            // https://stackoverflow.com/questions/56865217/php-curl-error-http-2-stream-0-was-not-closed-cleanly-protocol-error-err-1
            curl_setopt($ch, CURLOPT_HTTP_VERSION, CURL_HTTP_VERSION_1_1);
            curl_exec($ch);
            $error=curl_error($ch);
            if($error){
                throw new ServerException('Error getting screen file from Formulatrix: '.$error);
            }
            curl_close($ch);
            @fclose($out);
            if(!@file_exists($zipFile)){
                throw new ServerException('Could not save downloaded screen to temporary location on server');
            }
            $zip=new ZipArchive();
            if(!@$zip->open($zipFile)){
                throw new ServerException('Could not open downloaded zip file');
            }
            $xmlFiles=[];
            $numFiles=$zip->numFiles;
            for($i=0;$i<$numFiles;$i++){
                $containedFile=$zip->getNameIndex($i);
                if('.xml'==substr($containedFile,-4)){
                    $zip->extractTo($tempDir, array($containedFile));
                    $cleanUp[]=rtrim($tempDir,'/').'/'.$containedFile;
                    $xmlFiles[]=rtrim($tempDir,'/').'/'.$containedFile;
                }
            }
            $zip->close();

            if(empty($xmlFiles)){
                throw new ServerException('Found no XML files in downloaded zip file. Cannot add screen.');
            } else if(1!==count($xmlFiles)){
                throw new ServerException('Found more than one XML file in downloaded zip file. Cannot add screen.');
            }

            $screen=screen::create(array(
                'name'=>$request['name'],
                'manufacturer'=>$request['manufacturer'],
                'catalognumber'=>$request['catalognumber'],
                'isstandard'=>true,
                'tempfile'=>$xmlFiles[0],
                'mimetype'=>'text/xml'
            ));
            return $screen;

        } catch (Exception $e){
            throw $e;
        } finally {
            $imageStore=rtrim(config::get('core_imagestore'),'/');
            foreach ($cleanUp as $path){
                @unlink($path);
                while(rtrim($path,'/')!=rtrim($imageStore,'/')){
                    @rmdir($path);
                    $parts=explode('/', $path, -1);
                    $path=implode('/',$parts);
                }

            }
        }

    }

    /**
     * @return array
     * @throws ServerException
     */
    private static function getRockMakerScreens(){
        $url='https://formulatrix.com/protein-crystallization-systems/rock-maker-crystallization-software/';
        $ch = curl_init($url);
        $options = array(
            CURLOPT_COOKIEFILE => '',
            CURLOPT_FOLLOWLOCATION => true,
            CURLOPT_RETURNTRANSFER  => true,
            CURLOPT_HEADER          => false,
            CURLOPT_CONNECTTIMEOUT  => 10,
            CURLOPT_TIMEOUT         => 20,
            CURLOPT_SSL_VERIFYHOST  => 0,
            CURLOPT_SSL_VERIFYPEER  => false,
            CURLOPT_UNRESTRICTED_AUTH => true
        );
        curl_setopt_array($ch,$options);
        $data = curl_exec($ch);
        if(false===$data){
            throw new ServerException('Could not retrieve list of screens. ('.curl_error($ch).')');
        }

        $data=substr($data, stripos($data,'Download ROCK MAKER XML Screen Files'));
        $data=(explode('Our intent is to make ROCK MAKER XML',$data))[0];

        $parts=explode('<span class="pp-accordion-button-label" itemprop="name description">', $data);
        array_shift($parts);
        $manufacturers=[];
        foreach($parts as $manufacturer){
            $manufacturerName=substr($manufacturer,0, stripos($manufacturer, '</span>'));
            $tableRows=explode('<tr>',$manufacturer);
            $manufacturerAndScreens=array(
                'manufacturer'=>$manufacturerName
            );
            if(1==count($tableRows)){
                if(stripos($manufacturer,'href=')) {
                    $url=(explode('href="', $manufacturer))[1];
                    $url=(explode('"', $url))[0];
                    $manufacturerAndScreens['url']=$url;
                } else {
                    $manufacturerAndScreens['message']='No screens available';
                }
            } else {
                array_shift($tableRows);
                $blank=json_decode('"\u00a0"');
                $screens=[];
                foreach($tableRows as $tr){
                    $tr=str_replace($blank,'',urldecode($tr));
                    $tr=str_replace('&nbsp;','',$tr);
                    $tr=str_replace('</tr>','',$tr);
                    $tr=str_replace('<strong>','',$tr);
                    $tr=str_replace('</strong>','',$tr);
                    $tr=preg_replace('/<br.*>/U','',$tr);
                    $tr=preg_replace('/<td.*>/U','',$tr);
                    $tr=preg_replace('/<\/?span.*>/U','',$tr);
                    $tr=str_replace("\n",'',$tr);
                    $cells=array_slice(explode('</td>',$tr), 0, -1); //trailing </td> causes bogus ""
                    if(4==count($cells)){
                        if(''!=$cells[0]){
                            $screens[]=array(
                                'manufacturer'=>$manufacturerName,
                                'catalognumber'=>'',
                                'name'=>trim($cells[0]),
                                'path'=>static::getScreenPathFromHtmlLink($cells[1])
                            );
                        }
                        if(''!=$cells[2]){
                            $screens[]=array(
                                'manufacturer'=>$manufacturerName,
                                'catalognumber'=>'',
                                'name'=>trim($cells[2]),
                                'path'=>static::getScreenPathFromHtmlLink($cells[3])
                            );
                        }
                    } else if(6==count($cells)){
                        if(''!=trim($cells[0])){
                            $screens[]=array(
                                'manufacturer'=>$manufacturerName,
                                'catalognumber'=>trim($cells[0]),
                                'name'=>trim($cells[1]),
                                'path'=>static::getScreenPathFromHtmlLink($cells[2])
                            );
                        }
                        if(''!=trim($cells[3])){
                            $screens[]=array(
                                'manufacturer'=>$manufacturerName,
                                'catalognumber'=>trim($cells[3]),
                                'name'=>trim($cells[4]),
                                'path'=>static::getScreenPathFromHtmlLink($cells[5])
                            );
                        }
                    }
                }
                usort($screens, function ($item1, $item2) {
                    return strtolower($item1['name']) <=> strtolower($item2['name']);
                });
                $manufacturerAndScreens['screens']=$screens;
            }
            $manufacturers[]=$manufacturerAndScreens;
        }
        curl_close($ch);
        usort($manufacturers, function ($item1, $item2) {
            return strtolower($item1['manufacturer']) <=> strtolower($item2['manufacturer']);
        });
        return $manufacturers;
    }

    private static function getScreenPathFromHtmlLink($downloadLink){
        $path=(explode('href="', $downloadLink))[1];
        $path=(explode('"', $path))[0];
        $path=str_replace(static::$baseUri,'',$path);
        return $path;
    }

    public static function getById($id){
        throw new ServerException('not implemented');
    }

    public static function getByName($name){
        throw new ServerException('not implemented');
    }

    public static function update($id, $request=array()){
        throw new ServerException('not implemented');
    }

    public static function getByProperties($keyValuePairs, $request=array()){
        throw new ServerException('not implemented');
    }

    public static function getByProperty($key, $value, $request=array()){
        throw new ServerException('not implemented');
    }

    public static function delete($id){
        throw new ServerException('not implemented');
    }

}
